<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       http://AmorStyle
 * @since      1.0.0
 *
 * @package    Tweb3mmexplore
 * @subpackage Tweb3mmexplore/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Tweb3mmexplore
 * @subpackage Tweb3mmexplore/public
 * @author     Amor <AmorStyleYoga@Hotmail.com>
 */
class Tweb3mmexplore_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Tweb3mmexplore_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Tweb3mmexplore_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/tweb3mmexplore-public.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Tweb3mmexplore_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Tweb3mmexplore_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/tweb3mmexplore-public.js', array( 'jquery' ), $this->version, false );
	}

	//
	// This function is needed in order to be able to use the 'module' functionality of "import" and "export" that
	// is allowed in JavaScript. The id is not needed.
	//
	public function update_script_loader_tag($tag, $handle, $src)
	{
		// This overwrites the default <script> section in the header for this particular file.
		//echo "tag: ".$tag." handle:".$handle." src".$src."<br>";
		if ( 'tweb3mmexplore' === $handle ) {
			$tag = '<script type="module" src="' . esc_url( $src ) . '" id="js_test_module"></script>';
	    }
 
	    return $tag;
	}

}
