/*
provides support for send functionality.

The send functionality has four elements:
- user input of amount
- user input of address
- send button
- status

The send button is enabled when metamask is connected and there is an account active.
After the button is pressed, the button status changes to disabled (so it won't get pressed
again until the metamask call has returned.) If there is an error in the Metamask processing,
the button will be enabled and the status controll will get the error (if it is present).

The amount and address are read from the input controls when the button is pressed.

The status controll will be empty at the begining. Will change to "pending" after the
button has been pressed but before Metamask returns with an answer. When Metamask returns
the control is updated with the response: success or error. If the button is pressed and
the user input is found to be invalid, this controll will state the error.
*/
import * as dom from './domutils.js'

// These are the names of the three items we're going to look for on the page.
// Each item will be defened based on it's existance. 
const sendToButtonStr = "sendToButtonId"
const sendToStatusIdStr = "sendToStatusId"

const sendToAmountIdStr = "sendToAmountId"
const sendToIdStr = "sendToId"


class csendtobutton {

    #sendtoButton

    #sendToStatusIdDiv
    #sendToStatusMsg

    #amountIdDiv
    #amountValueWei
    #sendToIdDiv
    #sendToAddress

    #turnOnDebug

    constructor() {
        this.#sendtoButton = null

        this.#sendToStatusIdDiv = null
        this.#sendToStatusMsg = ""

        this.#amountIdDiv = null
        this.#amountValueWei = 0
        this.#sendToIdDiv = null
        this.#sendToAddress = 0

        this.#turnOnDebug = false;
        this.#DebugSpew("csendtobutton constructor")
    }

    turnOnDebug(b=false) {
        this.#turnOnDebug = b
    }

    #DebugSpew(Str) {
        if (true == this.#turnOnDebug) {
            console.log(Str)
        }
    }


    initHTML() {
        this.#DebugSpew("sendto.#initHTML() enter")

        if (null != this.getButton()) {
            this.#sendtoButton.innerText = 'Send'
            this.#sendtoButton.disabled = false
        }
        if (null == this.#sendToStatusIdDiv ) {
            this.#sendToStatusIdDiv = dom.fetch(this.#sendToStatusIdDiv, sendToStatusIdStr)
            this.setStatus("")
        }

        this.#DebugSpew("sendto.#initHTML() leave")
    }
    // fetches the button and returns true or false based on finding it.
    exists() {
        //if (this.getButton() != null) {
        //    this.#DebugSpew("sendto.exists() returning true")
        //}
        return (this.getButton() != null)
    }
    getButton() {
        if (null == this.#sendtoButton) {
            this.#sendtoButton = dom.fetch(this.#sendtoButton, sendToButtonStr)
        }
        return this.#sendtoButton
    }


    // returns the user's string
    getAmount() {
        //this.#DebugSpew("sendto.getAmount() entering")
        if (null == this.#amountIdDiv) {
            this.#amountIdDiv = dom.fetch(this.#amountIdDiv, sendToAmountIdStr)
        }
        //this.#DebugSpew("sendto.getAmount() leaving, returning: " + this.#amountIdDiv.value)
        return this.#amountIdDiv.value
    }
    getSendTo() {
        if (null == this.#sendToIdDiv) {
            this.#sendToIdDiv = dom.fetch(this.#sendToIdDiv, sendToIdStr)
        }
        return this.#sendToIdDiv.value
    }


    setStatus(Str) {
        this.#sendToStatusMsg = Str
        if (dom.exists(this.#sendToStatusIdDiv, sendToStatusIdStr)) {
            this.#sendToStatusIdDiv.innerHTML = this.#sendToStatusMsg
        }
    }
    getStatus() {
        return this.#sendToStatusMsg;
    }

    // Now, send specific functions here.
    //
    // Process: When the "Send" button is pressed, the website asynchronously calls
    // Metamask asking for a send opperation. That opperation will have a termination
    // condition. Thus, we're going to deactivate the send button until we've come
    // to a resolution to the request.
    //
    setSendInProgress() {
        this.setStatus("Send in progress")
        this.#DebugSpew("BUTTON IS BEING DISABLED")
        this.#sendtoButton.disabled = true
    }
    setSendComplete(send) {
        this.setStatus("Send Complete. Result: " + send)
        this.#DebugSpew("setSendComplete() success: The Sended Data: " + send)
        this.#DebugSpew("BUTTON IS BEING ENABLED")
        this.#sendtoButton.disabled = false
    }
    setSendCompleteErr(send) {
        this.setStatus("Send Complete. Error: " + send)
        this.#DebugSpew("setSendCompleteErr() failure")
        this.#DebugSpew("BUTTON IS BEING ENABLED")
        this.#sendtoButton.disabled = false
    }
    setSendInputErr(_errorStr) {
        this.setStatus("setSendInputErr() the Error: " + _errorStr)
        this.#DebugSpew("BUTTON IS BEING ENABLED")
        this.#sendtoButton.disabled = false
    }


/*
    Want to send 2tfuel (0x1BC16D674EC80000) to 0x077cCDb23328E84AD66d12f1a00B06d7b1321E33
    state that the gas price is "4000000000000"

request({
      method: 'eth_sendTransaction',
      params: [
        {
          from: accounts[0],
          to: '0x2f318C334780961FB129D2a6c30D0763d9a5C970',
          value: '0x29a2241af62c0000',
          //gasPrice: '0x09184e72a000',
          //gas: '0x2710',
          chainid: '0x016d' 
        },
      ],
    }

    chainids
    361 = 0x0169 = thetaNetId
    365 = 0x016d = thetaTestNetId
*/
    //
    // This routine returns the parameters needed to make an eth_sendtransaction
    // in the calling code.
    //
    getParams(_fromAddrHex, _sendToAddrHex, _amountToSendHex ) {
        this.#DebugSpew("sendto.getParams() Build params here.")

         const msgParams = {
                from: _fromAddrHex,
                to: _sendToAddrHex,
                value: _amountToSendHex,
                //gasPrice: '0x03A352944000',
                //gas: '0x2710',
                //chainid: '0x016d',
            }
        
        this.#DebugSpew(msgParams)
        return msgParams;
    }
}

export { csendtobutton }