import * as dom from './domutils.js'

// Mainnet is 361, testnet is 365
const thetaNetId = 361
const thetaTestNetId = 365

// string declarations
const signTypedData4IdStr = "signTypedData4Id"
const signTypedDataV4ResultIdStr = "signTypedDataV4ResultId"
const signTypedDataV4VerifyIdStr = "signTypedDataV4VerifyId"
const signTypedDataV4VerifyResultIsStr = "signTypedDataV4VerifyResultId"
// shows the state of the signing div
const signTypedDataV4StatusIdStr = "signTypedDataV4StatusId"

// for the signing process
const signTypedDataV4MsgIdStr = "signTypedDataV4MsgId"
const signTypedDataV4SecretIdStr = "signTypedDataV4SecretId"

//
// This class requires that the following two elements are present on the page:
//
// a button with id="signTypedData4Id"
// a span with id="signTypedDataV4ResultId"
//
// The results element can be hidden.
//
// When both these elements are present, the exist() function will return true.
// The class shoudl only be called when exist() returns true. 
//
class csignButtonV4 {

    #sbDiv
    #sbrDiv
    #vbDiv
    #vbrDiv

    #statusDiv

    #signMsgDiv
    #signSecretDiv

    #account
    #chainId

    #signInProgress
    #signSuccess


    #msgParams

    #turnOnDebug

    constructor() {
        // Can't access any element on the page.
        this.#sbDiv = null
        this.#sbrDiv = null
        this.#vbDiv = null
        this.#vbrDiv = null
        this.#statusDiv = null
        this.#signMsgDiv = null
        this.#signSecretDiv = null

        this.#account = null
        this.#chainId = 0

        this.#signInProgress = false
        this.#signSuccess = false


        this.#turnOnDebug = false;
        this.#DebugSpew("csignButton constructor")
    }

    turnOnDebug(b=false) {
        this.#turnOnDebug = b
    }

    #DebugSpew(Str) {
        if (true == this.#turnOnDebug) {
            console.log(Str)
        }
    }

    #initHTML() {
        this.getsignButton()
        this.getsignResults()

        if (null == this.#vbDiv) {
            this.#vbDiv = dom.fetch(this.#vbDiv, signTypedDataV4VerifyIdStr)
        }
        if (null == this.#vbrDiv ) {
            this.#vbrDiv = dom.fetch(this.#vbrDiv, signTypedDataV4VerifyResultIsStr)
        }

        if (null == this.#statusDiv ) {
            this.#statusDiv = dom.fetch(this.#statusDiv, signTypedDataV4StatusIdStr)
        }

        if (null == this.#signMsgDiv ) {
            this.#signMsgDiv = dom.fetch(this.#signMsgDiv, signTypedDataV4MsgIdStr)
        }
        if (null == this.#signSecretDiv ) {
            this.#signSecretDiv = dom.fetch(this.#signSecretDiv, signTypedDataV4SecretIdStr)
        }
    }

    // fetches the button and returns true or false based on finding it. This class should only be 
    // called when exists returns true.
    exists() {
        return ((this.getsignButton() != null) && (this.getsignResults() != null))
    }
    getsignButton() {
        if (null == this.#sbDiv) {
            this.#sbDiv = dom.fetch(this.#sbDiv, signTypedData4IdStr)
        }

        return this.#sbDiv
    }
    getsignResults() {
        if (null == this.#sbrDiv ) {
            this.#sbrDiv = dom.fetch(this.#sbrDiv, signTypedDataV4ResultIdStr)
        }
        return this.#sbrDiv
    }


    // if the class is not configured enough to make a signing call, it will return false.
    ready() {
        if (this.#account && this.#account.length > 0 ) {
            if (this.#chainId == thetaNetId ) {
                this.#DebugSpew("sb.ready() returning true")
                return true;
            }
        }
        this.#DebugSpew("sb.ready() returning false")
        return false;
    }

    #readyReason() {
        let reason = ""
        if (null == this.#account ) {
            reason += " No Account Number;"
        }
        else {
            // this.#account is not null
            if (0 == this.#account.length) {
                reason += " No Active Account;"
            }

            if (this.#chainId != thetaNetId ) {
                reason += " Not connected to Theta Network;"
            }
        }
        if (0 == reason.length) {
            if (this.#signInProgress) {
                reason += "Signing in Progress"
            }
            else {
                reason += "Active"
            }
        }
        return reason;
    }

    updateState() {

        if (this.#signInProgress )
        {
            this.#DebugSpew("sb.updateState() ready")
            this.#sbDiv.disabled = true
            this.#sbDiv.innerText = this.#readyReason()

            if (null != this.#vbDiv ) {
                //this.#disableVerifyButton()
                this.#vbDiv.disabled = true
                this.#vbDiv.innerText = "Verify - disabled"
            }
            if (null != this.#statusDiv ) {
                this.#statusDiv.style.display = ""
                this.#statusDiv.innerText = this.#readyReason()
            }
        }
        else {
            if (this.ready()) {
                this.#DebugSpew("sb.updateState() ready")
                // Now reset both sign and verify buttons
                //this.#enableSignButton()
                this.#sbDiv.disabled = false
                this.#sbDiv.innerText = "Sign"

                if (null != this.#vbDiv ) {
                    //this.#disableVerifyButton()
                    this.#vbDiv.disabled = true
                    this.#vbDiv.innerText = "Verify - disabled"
                }
                // if signing has occurred.
                if (null != this.#statusDiv ) {
                    this.#statusDiv.style.display = ""
                    this.#statusDiv.innerText = this.#sbrDiv.innerText
                }
            }
            else {
                this.#DebugSpew("sb.updateState() not ready")
                this.#sbDiv.disabled = true
                this.#sbDiv.innerText = "Sign - disabled"

                if (null != this.#vbDiv ) {
                    //this.#disableVerifyButton()
                    this.#vbDiv.disabled = true
                    this.#vbDiv.innerText = "Verify - disabled"
                }
                if (null != this.#statusDiv ) {
                    this.#statusDiv.style.display = ""
                    this.#statusDiv.innerText = this.#readyReason()
                }
            }
        }
    }

    clearResults() {
        if (null != this.#sbrDiv ) {
            this.#sbrDiv.innerText = ""
        }
        if (null != this.#vbrDiv ) {
            this.#vbrDiv.innerText = ""
        }

        if (null != this.#statusDiv ) {
            this.#statusDiv.innerText = this.#readyReason()
        }
    }


    // Can be called at any time, thus it inits the HTML.
    handleChainIdChange(chainId) {
        this.#initHTML()

        if (this.#chainId != chainId )
        {
            this.#DebugSpew("sb.handleChainIdChange() changing")
            // the chainId changed! all the data we've gathered becomes invalid
            this.#chainId = chainId
            this.clearResults()
            this.#setmsgParams()
        }
        this.updateState()
    }
    getChainId() {
        return this.#chainId
    }

    // Can be called at any time, thus it inits the HTML.
    handleAccountChange(account) {
        this.#initHTML()

        if (this.#account != account ) {
            this.#DebugSpew("sb.handleAccountChange() changing")
            // the account changed! all the data we've gathered becomes invalid
            this.#account = account
            this.clearResults()
        }
        this.updateState()
    }
    getAccount() {
        return this.#account[0]
    }


    // Signing state changes
    setSignInProgress() {
        this.#signInProgress = true;
        this.#signSuccess = false;
        this.updateState()
    }
    setSign(sign) {
        this.#signInProgress = false;
        this.#signSuccess = true;
        if (null != this.#sbrDiv ) {
            this.#sbrDiv.innerText = sign
        }
        this.#DebugSpew("setSign() success")
        this.#DebugSpew("The Signed Data: "+ sign)
        // Now update the buttons
        this.updateState()
    }
    setSignErr(sign) {
        this.#signInProgress = false;
        if (null != this.#sbrDiv ) {
            this.#sbrDiv.innerText = sign
        }
        this.#DebugSpew("setSign() failure")
        this.updateState()
    }

    // For signing
    #setmsgParams() {

        this.#DebugSpew("#setmsgParams() enter")

        let theMsg;
        let theSecret;

        if (null != this.#signMsgDiv) {
            theMsg = this.#signMsgDiv.innerText
        } 
        else {
            theMsg = "Generic Message"
        }
        if (null != this.#signSecretDiv) {
            theSecret = this.#signSecretDiv.innerText
        }
        else {
            theSecret = "Generic Secret"
        }

        this.#msgParams = {
            domain: {
                chainId: this.#chainId.toString(),
                name: 'AmorStyle Pluggin Project',
                version: '1',
            },
            message: {
                contents: theMsg, // Generated from the website
                secret: theSecret, // Generated from the website
            },
            primaryType: 'message',
            types: {
                EIP712Domain: [
                    { name: 'name', type: 'string' },
                    { name: 'version', type: 'string' },
                    { name: 'chainId', type: 'uint256' },
                ],
                message: [
                    { name: 'contents', type: 'string' },
                    { name: 'secret', type: 'string' },
                ],
            }
        }

        if (true == this.#turnOnDebug) {
            console.dir(this.#msgParams)
        }
        this.#DebugSpew("#setmsgParams() leave")
    }
    getParams() {
        this.#setmsgParams()
        return JSON.stringify(this.#msgParams)
    }

}

export { csignButtonV4 }

/*
        this.#msgParams = {
            domain: {
                chainId: this.#chainId.toString(),
                name: 'Ether Mail',
                verifyingContract: '0xCcCCccccCCCCcCCCCCCcCcCccCcCCCcCcccccccC',
                version: '1',
            },
            message: {
                contents: 'Hello, Bob!',
                from: {
                    name: 'Cow',
                    wallets: [
                        '0xCD2a3d9F938E13CD947Ec05AbC7FE734Df8DD826',
                        '0xDeaDbeefdEAdbeefdEadbEEFdeadbeEFdEaDbeeF',
                    ],
                },
                to: [
                    {
                        name: 'Bob',
                        wallets: [
                            '0xbBbBBBBbbBBBbbbBbbBbbbbBBbBbbbbBbBbbBBbB',
                            '0xB0BdaBea57B0BDABeA57b0bdABEA57b0BDabEa57',
                            '0xB0B0b0b0b0b0B000000000000000000000000000',
                        ],
                    },
                ],
            },
            primaryType: 'Mail',
            types: {
                EIP712Domain: [
                    { name: 'name', type: 'string' },
                    { name: 'version', type: 'string' },
                    { name: 'chainId', type: 'uint256' },
                    { name: 'verifyingContract', type: 'address' },
                ],
                Group: [
                    { name: 'name', type: 'string' },
                    { name: 'members', type: 'Person[]' },
                ],
                Mail: [
                    { name: 'from', type: 'Person' },
                    { name: 'to', type: 'Person[]' },
                    { name: 'contents', type: 'string' },
                ],
                Person: [
                    { name: 'name', type: 'string' },
                    { name: 'wallets', type: 'address[]' },
                ],
            }
        }
*/