
//
// ********************************************************************************************************
//
// This section is a roll-your-own parsing of 18 digit numbers into humanly readable values. The class
// will allow for parsing out information that fits Wei type numbers. It also stores only the hex number
// that it was originally created with.
//
// Each of these methods return a string.
//
// .units(_bformat = false) -> this routine returns a string that represents the units, if the 
//                             parameter is true, the number will be formatted with commas.
//
// .full(_bformat = false, _digits = 18, _clip - false) -> this routien returns a string that represents the entire value.
//                            If the user wants it formatted, they get all the digits
// Parameters:
//
// - _bFormat -> Add commas and decimal point to make the number human readable
// - _digits  -> Only show this many digits. They are clipped rather than rounded.
// - _clip    -> Remove trailing zeros (will always display at least 1 decimal place x.y)
//
// The default constructor initializes the class to a hex number of zero. 
// 
export default class CWeiNumber {

    #m_theNumInWei; // raw from server is in hex

    // number comes in as hex number from the API
    constructor(_theNumInWei = "0x0") {
        this.set(_theNumInWei)
    }

    raw() {
        return this.#m_theNumInWei
    }

    #strToHex(Str) {
        return "0x" + BigInt(Str).toString(16);
    }

    // The set routine accepts either a hex number or a base 10 number. The default is hex number
    set(_theNumInWei, _dec = false) {
        if (_dec) {
            // base 10 number
            this.#m_theNumInWei = this.#strToHex(_theNumInWei)
            //console.log("m_theNumInWei: " + this.#m_theNumInWei)
        }
        else {
            // hex number
            this.#m_theNumInWei = _theNumInWei;
            //console.log("m_theNumInWei: " + this.#m_theNumInWei)
        }
        //console.log("Balance: " + this.full(true, 2, true) + " (" + this.#m_theNumInWei + " hex wei)")
    }

    // The algorithm for formatting the number will be
    // - convert number into string
    // - reverse the number
    // - move characters into destination location observing '.' and ',' locations.
    // - save units number.

    // reverses the string.
    #reverse(s) {
        let l = s.length
        var o = '';
        for (var i = s.length - 1; i >= 0; i--) {
            o += s[i];
        }
        return o;
    }

    // parse out the units and return a number as a string
    units(_bformat = false) {

        let bigStr = BigInt(this.#m_theNumInWei).toString(10)

        let l = bigStr.length
        if (l <= 18) {
            return "0"; // there are no units
        }
        else {
            // reverse the string
            //console.log("bigStr " + bigStr)
            let rStr = this.#reverse(bigStr)

            let loc = 0;
            var o = '';
            for (var i = 18; i < l; i++ , loc++) {

                if (_bformat) {
                    // means, add commas as appropriate
                    if (loc == 3) { o += ','; i--; }
                    else if (loc == 7) { o += ','; i--; }
                    else if (loc == 11) { o += ','; i--; }
                    else if (loc == 15) { o += ','; i--; }
                    else if (loc == 19) { o += ','; i--; }
                    else {
                        o += rStr[i];
                    }
                }
                else {
                    o += rStr[i];
                }
            }

            let Str = this.#reverse(o)
            //console.log(Str)
            return Str
        }
    }

    full(_bformat = false, _digits = 18, _clip = false) {

        //let local = this.#m_theDecInWei
        let bigStr = BigInt(this.#m_theNumInWei).toString(10)
        if (!_bformat) {
            return bigStr
        }

        //console.log("bigStr " + bigStr)
        //let Str = (this.#m_theDecInWei).toString
        let rStr = this.#reverse(bigStr)
        //console.log("- Str: " + bigStr + " rStr: "+ rStr + " _digits: " + _digits)

        // if we're smaller than 18,
        let l = rStr.length
        if (l <= 18) {
            // Need to zero fill the result to 0.xxx or 0x00xxx in order to make sure that
            // we always return a number of that is 0.18digits
            let loc = 0;
            var o = '';
            for (var i = 0; i < (rStr.length); i++ , loc++) {
                o += rStr[i];
            }
            // now for the padding
            for (var i = loc; i < 18; i++ , loc++) {
                o += '0';
            }
            o += '.';
            o += '0';
        }
        else {
            let loc = 0;
            var o = '';
            for (var i = 0; i < (rStr.length); i++ , loc++) {

                // if we get to decimal or comma values, add that char
                if (loc == 18) { o += "."; i--; }
                else if (loc == 22) { o += ","; i--; }
                else if (loc == 26) { o += ","; i--; }
                else if (loc == 30) { o += ","; i--; }
                else if (loc == 34) { o += ","; i--; }
                else if (loc == 38) { o += ","; i--; }
                else {
                    o += rStr[i];
                }
            }
        }

        //console.log(o)
        let Str = this.#reverse(o)
        let len = Str.length;
        let outStr = "";
        if (_digits < 18 && len > 1) {

            //console.log("clip to: "+_digits +" Str: "+ Str)
            // Now clip out any digits that are not wanted by the caller
            let iDigits = 0;
            let bFound = false;
            for (var i = 0; i < len; i++) {
                if (Str[i] == '.') {
                    bFound = true;
                    if (_digits > 0) {
                        outStr += Str[i];
                    }
                    continue;
                }
                if (bFound) {
                    if (iDigits < _digits) {
                        outStr += Str[i];
                        iDigits++;
                    }
                }
                else {
                    outStr += Str[i];
                }
            }
            //return outStr;
        }
        else {
            outStr = Str;
        }

        let outResult = "";
        if (_clip) {

            //console.log("Need to clip zeros: " + outStr)
            // now if the user wants to clip trailing zeros
            // reverse string
            // skip leading zeros when moving chars
            // flip string.
            let rStr = this.#reverse(outStr)
            let len = rStr.length;

            let Str = ""
            let bNum = false;
            let bDec = true;
            for (var i = 0; i < len; i++) {

                if (bDec) {
                    if ('0' == rStr[i]) {
                        if (bNum) {
                            Str += rStr[i];
                        }
                    }
                    else {
                        if ('.' == rStr[i]) {
                            bDec = false
                            if (!bNum) {
                                Str += '0';
                            }
                            Str += '.';
                        }
                        else {
                            Str += rStr[i];
                            bNum = true;
                        }
                    }
                }
                else {
                    Str += rStr[i];
                }

            }

            outResult = this.#reverse(Str);
        }
        else {
            outResult = outStr;
        }

        return outResult;
    }

    // returns the unformated full version 
    wei() {
        return this.full()
    }
    coin() {
        return this.units()
    }
}
